<?php

namespace Shikiryu\LBCReposter;

use Symfony\Component\DomCrawler\Crawler;

class Actions
{
    const HOME_URL = 'https://www.leboncoin.fr/';
    const LOGIN_URL = 'https://www.leboncoin.fr/beta/ajax/popins/connexion.html';
    const ACCOUNT_URL = 'https://compteperso.leboncoin.fr/account/index.html';
    const ADD_URL = 'https://www.leboncoin.fr/ai/form/0';
    const DELETE_URL = 'https://compteperso.leboncoin.fr/store/main?cmd=adservices';
    const UPLOAD_URL = 'https://www.leboncoin.fr/ai/photo_upload_ajax/0';

    /**
     * @var Account
     */
    protected $account;

    /**
     * Actions constructor.
     * @param Account $account
     */
    public function __construct(Account $account)
    {
        $this->account = $account;
    }


    /**
     * Check if connected, if not, try to connect you
     *
     * @see Account#isConnected
     *
     * @return bool
     */
    public function connect()
    {
        if (!$this->account->isConnected()) {
            // Let's connect to your account (or not)
            $crawler    = $this->account->getClient()->request('GET', self::LOGIN_URL);
            $form       = $crawler->selectButton('Se connecter')->form();
            $crawler    = $this->account->getClient()->submit($form, ['st_username' => $this->account->getConfig()->login, 'st_passwd' => $this->account->getConfig()->password]);
            $this->account->setConnected($crawler->filter('.account_userinfo')->count() > 0);
        }
        return $this->account->isConnected();
    }

    /**
     * Retrieve all deals from the current account
     *
     * @return Deals
     */
    public function retrieve()
    {
        if ($this->connect()) {
            // Let's go to our dashboard
            $crawler    = $this->account->getClient()->request('GET', self::ACCOUNT_URL);
            // Let's list our existing deals
            $deals      = $crawler->filter('#dashboard .list .element')->each(
                function (Crawler $node) {
                    return $node->filter('.detail .title')->each(
                        function (Crawler $n) {
                            return $n->filter('a')->first()->attr('href');
                        }
                    );
                }
            );
            // Let's flatten that array
            $deals = array_map('current', $deals);
            return (new Deals())->setAccount($this->account)->setDeals($deals);
        }
        // Let's return empty deals container
        return (new Deals())->setAccount($this->account);
    }

    /**
     * Create the given deal in the account
     *
     * @param Deal $deal
     *
     * @return bool
     */
    public function create(Deal $deal)
    {
        try {
            $crawler = $this->account->getClient()->request('GET', self::ADD_URL);
            $form = $crawler->selectButton('Valider')->form();
            $image0 = sprintf('%s/%s/image0.jpg', DEALS_DIR, $deal->getId());
            $fields = $form->getPhpValues();
            $fields = array_merge(
                $fields,
                [
                    'geo_source'            => 'user',
                    'geo_provider'          => 'lbc',
                    'latitude'              => '49.0707',
                    'longitude'             => '2.31882',
                    'accept_localisation'   => 'on',
                    'check_type_diff'       => '0',
                    'location_p'            => sprintf('%s %s', $this->account->getConfig()->city, $this->account->getConfig()->postal_code),
                    'zipcode'               => $this->account->getConfig()->postal_code,
                    'city'                  => $this->account->getConfig()->city,
                    'region'                => $this->account->getConfig()->region,
                    'dpt_code'              => $this->account->getConfig()->department,
                    'address'               => $this->account->getConfig()->address,
                    'name'                  => $this->account->getConfig()->name,
                    'email'                 => $this->account->getConfig()->login,
                    'phone'                 => $this->account->getConfig()->phone,
                    'category'              => $deal->getCategory(),
                    'type'                  => $deal->getType(),
                    'subject'               => $deal->getSubject(),
                    'body'                  => $deal->getBody(),
                    'price'                 => $deal->getPrice(),
                    'no_salesmen'           => 1,
                    'phone_hidden'          => 1,
                ]
            );
            $uri = $form->getUri();
            // It needs to be done twice !!
            $this->account->getClient()->request('POST', $uri, $fields, ['image0' => $image0]);
            $crawler = $this->account->getClient()->request('POST', $uri, $fields, ['image0' => $image0]);
            // TODO need to check if we're in the good page
            // Let's validate
            $form = $crawler->selectButton('Valider mon annonce')->form();
            $crawler = $this->account->getClient()->submit($form, ['accept_rule' => 1]);
            // TODO return if it's the validation page or not
            return true;
        } catch (\Exception $e) {
            // TODO manage exceptions
            echo $e->getTraceAsString();
            return false;
        }
    }

    /**
     * Delete the deal from the account
     *
     * @param Deal $deal
     *
     * @return bool
     */
    public function delete(Deal $deal)
    {
        $fields = [
            sprintf('selected_ads[%s]', $deal->getId())    => 'on',
            'cmd'                                                  => 'delete',
            'continue'                                            => 'Continuer'
        ];
        $crawler    = $this->account->getClient()->request('POST', self::DELETE_URL, $fields);
        // confirmation
        $form       = $crawler->selectButton('Valider')->form();
        $crawler    = $this->account->getClient()->submit($form, ['delete_reason' => '1']);
        // TODO return if it's the validation page or not
        return true;
    }
}